#ifndef MSP_IO_PRINT_H_
#define MSP_IO_PRINT_H_

#include <msp/strings/format.h>
#include "base.h"
#include "console.h"

namespace Msp {
namespace IO {

/**
Writes a string to an I/O object.  Same as o.write(f).  Provided for
completeness with the other print functions.
*/
inline unsigned print(Base &o, const std::string &f)
{ return o.write(f); }

#if __cplusplus>=201103L
template<typename... Args>
inline unsigned print(Base &o, const std::string &f, Args... args)
{ return print(o, format(f, args...)); }

#else
template<typename A1>
inline unsigned print(Base &o, const std::string &f, A1 a1)
{ return print(o, format(f, a1)); }

template<typename A1, typename A2>
inline unsigned print(Base &o, const std::string &f, A1 a1, A2 a2)
{ return print(o, format(f, a1, a2)); }

template<typename A1, typename A2, typename A3>
inline unsigned print(Base &o, const std::string &f, A1 a1, A2 a2, A3 a3)
{ return print(o, format(f, a1, a2, a3)); }

template<typename A1, typename A2, typename A3, typename A4>
inline unsigned print(Base &o, const std::string &f, A1 a1, A2 a2, A3 a3, A4 a4)
{ return print(o, format(f, a1, a2, a3, a4)); }

template<typename A1, typename A2, typename A3, typename A4, typename A5>
inline unsigned print(Base &o, const std::string &f, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5)
{ return print(o, format(f, a1, a2, a3, a4, a5)); }
#endif

/* The following functions print to console */

inline unsigned print(const std::string &f)
{ return print(cout, f); }

#if __cplusplus>=201103L
template<typename... Args>
inline unsigned print(const std::string &f, Args... args)
{ return print(cout, f, args...); }

#else
template<typename A1>
inline unsigned print(const std::string &f, A1 a1)
{ return print(cout, f, a1); }

template<typename A1, typename A2>
inline unsigned print(const std::string &f, A1 a1, A2 a2)
{ return print(cout, f, a1, a2); }

template<typename A1, typename A2, typename A3>
inline unsigned print(const std::string &f, A1 a1, A2 a2, A3 a3)
{ return print(cout, f, a1, a2, a3); }

template<typename A1, typename A2, typename A3, typename A4>
inline unsigned print(const std::string &f, A1 a1, A2 a2, A3 a3, A4 a4)
{ return print(cout, f, a1, a2, a3, a4); }

template<typename A1, typename A2, typename A3, typename A4, typename A5>
inline unsigned print(const std::string &f, A1 a1, A2 a2, A3 a3, A4 a4, A5 a5)
{ return print(cout, f, a1, a2, a3, a4, a5); }
#endif

} // namespace IO
} // namespace Msp

#endif
